from typing import Dict, List
from zipfile import ZipFile
from UniversalParser._tools import read_content_from_zipobj
import UniversalParser as UP
from .ContentTypes import UPContentTypes
from .Sheet import UPSheet

class UPWorkBook:

    def __init__(self, excel_path: str, mode: str = 'a') -> None:

        self.zip_obj = ZipFile(excel_path, mode=mode)

        self.contentTypes = UPContentTypes(self.zip_obj) # 全部的XML路径类型名称对照表

        self.main_path = self.contentTypes.get_main_path().split('/')

        self.Relationship: Dict[str, str] = {}
        self._init_Relationship()

        self._sis: List[str] = [] # 中文对照表
        try:
            self._init_sharedString()
        except:
            pass

        self.sheets: List[UPSheet] = []
        self._init_sheets()
    
    def _init_Relationship(self) -> None:
        p_dir = '/'.join(self.main_path[:-1])
        main_file_name = self.main_path[-1]
        _rels_path: str = f"{p_dir}/_rels/{main_file_name}.rels"
        content = read_content_from_zipobj(self.zip_obj, _rels_path)
        rels_manager = UP.parse_xml(content, analysis_text=False)
        for _r in rels_manager.find_nodes_by_tag('Relationship'):
            self.Relationship[_r.Id] = f"{p_dir}/{_r.Target}" # whole
            
    def _init_sharedString(self):
        shareString_path = self.contentTypes.get_shareString_path()
        content = read_content_from_zipobj(self.zip_obj, shareString_path)
        manager = UP.parse_xml(content, analysis_text=False)
        for _ in manager.find_nodes_by_tag('si'):
            self._sis.append(_.t)

    def _init_sheets(self):
        content = read_content_from_zipobj(self.zip_obj, '/'.join(self.main_path))
        manager = UP.parse_xml(content, analysis_text=False)
        for sheet in manager.find_nodes_by_tag('sheet'):
            sheet_path = self.Relationship[sheet['r:id']]
            self.sheets.append(
                UPSheet(sheet.name, sheet.sheetId, sheet['r:id'], self.zip_obj, sheet_path, self._sis)
            )

    def get_sheet_by_name(self, name: str) -> UPSheet:
        pass

    def add_sheet(self):
        pass

    def del_sheet(self):
        pass

    def __del__(self):
        try:
            self.zip_obj.close()
        except:
            pass



from typing import Any, List
from zipfile import ZipFile
from UniversalParser._tools import read_content_from_zipobj
import UniversalParser as UP
from ._tools import *

class UPSheet:

    def __init__(self
            , name: str
            , sheetId: str
            , r_id: str
            , zip_obj: ZipFile
            , sheet_path: str
            , sis: List[str]
        ) -> None:

        self.name = name
        self.sheetId = sheetId
        self.r_id = r_id
        self._sis = sis
        
        self.shape = [1, 1] # 维度信息，[行数, 列数]
        self.size = 0 # 元素个数

        content = read_content_from_zipobj(zip_obj, sheet_path)
        self.manager = UP.parse_xml(content, analysis_text=False)

        self._init_shape()
    
    def _init_shape(self):
        dimension = self.manager.find_nodes_by_tag('dimension', one_=True)
        try:
            begin_loc, end_loc = dimension.ref.strip().split(':')
            begin_info = split_char_number(begin_loc)
            end_info = split_char_number(end_loc)
            self.shape[1] = get_width_by_char(begin_info[0], end_info[0])
            self.shape[0] = int(end_info[1]) - int(begin_info[1]) + 1
        except:
            pass

    def get_cell_by_coordinate(self, coordinate: str) -> Any:
        '''根据坐标 'A1'、'B23'等等定位获取单元格的值
        
        如果坐标对应的单元格的值为空，则一致返回 None
        '''
        try:
            cell = self.manager.find_node_by_attrs(r=coordinate)
        except:
            return None
        else:
            v = cell.get('v')
            if v is not None: v=self.manager.find_text(v)
            if 't' in cell and 's' == cell.t:
                v = self._sis[int(v)]
            return v if v else None

    def modify_cell_by_coordinate(self, coordinate: str):
        pass

    def modify_name(self, new_name: str) -> str:
        pass



class CoreData:
    RELS_FOLDER_NAME = '_rels'

class ContentTypeOverride:
    EXTENDED_PROPERTIES = [
        'application/vnd.openxmlformats-officedocument.extended-properties+xml',
    ]

    CORE_PROPERTIES = [
        'application/vnd.openxmlformats-package.core-properties+xml',
    ]

    CUSTOM_PROPERTIES = [
        'application/vnd.openxmlformats-officedocument.custom-properties+xml',
    ]

    SHAREDSTRINGS = [
        'application/vnd.openxmlformats-officedocument.spreadsheetml.sharedStrings+xml',
    ]
    
    STYLES = [
        'application/vnd.openxmlformats-officedocument.spreadsheetml.styles+xml',
    ]

    THEME = [
        'application/vnd.openxmlformats-officedocument.theme+xml',
    ]

    MAIN = [
        'application/vnd.openxmlformats-officedocument.spreadsheetml.sheet.main+xml',
        'application/vnd.ms-excel.sheet.macroEnabled.main+xml',
    ]

    WORKSHEET = [
        'application/vnd.openxmlformats-officedocument.spreadsheetml.worksheet+xml',
    ]



from typing import Type
from zipfile import ZipFile
from UniversalParser._tools import read_content_from_zipobj
import UniversalParser as UP
from .constant import ContentTypeOverride

class UPContentTypes:

    PATH = '[Content_Types].xml'

    def __init__(self, zip_obj: ZipFile) -> None:
        content = read_content_from_zipobj(zip_obj, self.PATH)
        self.manager = UP.parse_xml(content, analysis_text=False)

    def get_main_path(self) -> str:
        for content_type in ContentTypeOverride.MAIN:
            try:
                return self.manager.find_node_by_attrs(ContentType=content_type).PartName.strip('/')
            except:
                pass
        else:
            raise TypeError('Unsupport. Please contact the developer.')

    def get_app_path(self) -> str:
        for content_type in ContentTypeOverride.EXTENDED_PROPERTIES:
            try:
                return self.manager.find_node_by_attrs(ContentType=content_type).PartName.strip('/')
            except:
                pass
        else:
            raise TypeError('Unsupport. Please contact the developer.')

    def get_core_path(self) -> str:
        for content_type in ContentTypeOverride.CORE_PROPERTIES:
            try:
                return self.manager.find_node_by_attrs(ContentType=content_type).PartName.strip('/')
            except:
                pass
        else:
            raise TypeError('Unsupport. Please contact the developer.')

    def get_custom_path(self) -> str:
        for content_type in ContentTypeOverride.CUSTOM_PROPERTIES:
            try:
                return self.manager.find_node_by_attrs(ContentType=content_type).PartName.strip('/')
            except:
                pass
        else:
            raise TypeError('Unsupport. Please contact the developer.')

    def get_shareString_path(self) -> str:
        for content_type in ContentTypeOverride.SHAREDSTRINGS:
            try:
                return self.manager.find_node_by_attrs(ContentType=content_type).PartName.strip('/')
            except:
                pass
        else:
            raise TypeError('Unsupport. Please contact the developer.')
            



import re
from typing import Optional, Tuple

patt = re.compile(r'([A-Z]+)([0-9]+)')

def split_char_number(s: str) -> Optional[Tuple[str, str]]:
    obj = patt.match(s)
    if obj:
        return obj.groups()

def _minus_char(char_begin: str, char_end: str) -> int:
    return ord(char_end) - ord(char_begin) + 1

def _26_to_10(s: str) -> int:
    _26, v = 0, 0
    for _c in s[::-1]:
        v += _minus_char('A', _c) * pow(26, _26)
        _26 += 1
    return v

def get_width_by_char(begin_s: str, end_s: str) -> int:
    return _26_to_10(end_s) - _26_to_10(begin_s) + 1

